//You can include any C libraries that you normally use
#include "math.h"
#include "mex.h"   //--This one is required
#include <vector>

#include "ann.cpp"
#include "bd_pr_search.cpp"
#include "bd_search.cpp"
#include "bd_tree.cpp"
#include "brute.cpp"
#include "kd_pr_search.cpp"
#include "kd_search.cpp"
#include "kd_split.cpp"
#include "kd_tree.cpp"
#include "kd_util.cpp"
#include "kd_dump.cpp"
#include "perf.cpp"


double distance2(double* a, double* b, int dim)
{
    //mexPrintf("a: "); for(int i=0; i!=dim; ++i) mexPrintf("%f ", a[i]); mexPrintf("\n");
    //mexPrintf("b: "); for(int i=0; i!=dim; ++i) mexPrintf("%f ", b[i]); mexPrintf("\n");
    double d = 0;
    for(int i=0; i!=dim; ++i) d += (a[i]-b[i])*(a[i]-b[i]);
    return d;    
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    // checks
    if(nrhs!=2 && nrhs!=3) {mexPrintf("Number of input must be 2 or 3\n"); return;}
    else { if(nrhs==2 && nlhs!=1) {mexPrintf("Number of output must be 1\n"); return;} else if(nrhs==3 && nlhs!=2) {mexPrintf("Number of output must be 2\n"); return;} }
    
    // Get X
    double* X        = mxGetPr (prhs[0]);
    int     nbPoints = mxGetM  (prhs[0]);
    int     nbDims   = mxGetN  (prhs[0]);

    // get K
    int     K        = (int)*mxGetPr(prhs[1]);
    
    // create output
    plhs[0]          = mxCreateDoubleMatrix(nbPoints, K, mxREAL);
    double* data     = mxGetPr (plhs[0]);
    for(int i=0; i!=nbPoints*K; ++i) data[i] = -1;

    // Build list of points
    ANNpointArray dataPts = annAllocPts(nbPoints,nbDims);
    ANNpoint      queryPt = annAllocPt(nbDims);
    ANNidxArray   nnIdx   = new ANNidx[K];
    ANNdistArray  dists   = new ANNdist[K];
    double        eps     = 0; // error bound
    
    for(int n=0; n!=nbPoints; ++n)
        for(int i=0; i!=nbDims; ++i) dataPts[n][i] = X[i*nbPoints + n];
    
    ANNkd_tree*   kdTree = new ANNkd_tree(dataPts, nbPoints, nbDims);

    // Iterate points and find K closest points
    for(int n=0; n!=nbPoints; ++n)
    {
        for(int i=0; i!=nbDims; ++i) queryPt[i] = X[i*nbPoints + n];
        
        // get the knn
        kdTree->annkSearch(queryPt, K, nnIdx, dists, eps);
        for(int k=0; k!=K; ++k) dists[k] = sqrt(dists[k]);
        
        // save knn
        for(int k=0; k!=K; ++k) data[k*nbPoints+n] = nnIdx[k];
    }
    
    // matlab uses +1 as indices
    for(int i=0; i!=nbPoints*K; ++i) ++data[i];

    // cleaning
    delete [] nnIdx;
    delete [] dists;
    delete kdTree;
    annClose();
}
