%% Create MAT file
%% Herve Lombaert, May 3rd, 2010
%%
%% Purpose: MAT files are much faster to load than text files
%%
%% X,F - mesh vertices and triangular faces
%% Parcels - Npoints x Nparcels, P(i,j) tells if the ith mesh node belong to parcel j
%% Sulcaldepth - Sulcal depth at the ith point
%% Thickness - Cortical thickness at the ith point
%% Curvature - White matter curvature at the ith point

clear all;

pathname = '../correspondence/data';
outpath = 'data';
ids    = [17,19,21,22,23,26,27,28,30,31,33,34,36,38,39,40];
studys = [1,1,1,2,2,3,3,2,3,1,2,1,1,2,2,2];
useright = 1;

for i=1:size(ids,2)
    
    id1    = ids(i);
    study1 = studys(i);
    disp(['Creating MAT file for ' num2str(id1,'%03d')]);
    
    %% filenames
    pathname_extra = [pathname '/MT_' num2str(id1,'%03d') '/' num2str(study1,'%02d') '_recon'];
    if useright; leftright = 'rh'; else leftright = 'lh'; end;

    %% Load mesh
    [X,F] = loadmesh([pathname '/MT_' num2str(id1,'%03d') '__' num2str(study1,'%02d') '_recon__surfs__v1_labels__sulc.mat'],useright);

    %% Load V1 area
    V1    = loadv1([pathname '/MT_' num2str(id1,'%03d') '__' num2str(study1,'%02d') '_recon__surfs__v1_labels__sulc.mat'],useright);
    
    %% Load parcels
    parcels     = [3:73 75:77 79:81];
    Parcels     = loadparcels([pathname_extra],size(X,1),parcels,useright);

    %% Sulcal depth
    Sulcaldepth = loadsulcaldepth([pathname_extra '/' leftright '.sulc.asc']);

    %% Cortical Thickness
    Thickness   = loadsulcaldepth([pathname_extra '/' leftright '.thickness.asc']);

    %% Curvature
    Curvature   = loadsulcaldepth([pathname_extra '/' leftright '.white.curv.asc']);

    %% Save into file
    if ~exist(outpath); mkdir(outpath); end;
    if useright; leftright = 'R'; else leftright = 'L'; end;
    save([outpath '/' num2str(id1,'%03d') leftright '.mat'],'X','F','V1','Sulcaldepth','Thickness','Curvature','Parcels');

%     %% Save FreeSurfers correspondence maps
%     corr_path = [outpath '/correspondence_freesurfer'];
%     if ~exist(corr_path); mkdir(corr_path); end;
%     for i=1:size(ids,2);
%         Correspondence = loadfreesurfermatching(pathname,id1,study1,ids(i),studys(i),useright);
%         save([corr_path '/' num2str(id1,'%03d') leftright '_' num2str(ids(i),'%03d') leftright '.mat'], 'Correspondence');
%     end
    
end
