%% loadfreesurfermatching.m
%% Herve Lombaert, April 28th, 2010
%%
%% Load FreeSurfers matching
%%
%% pathname   - where the MT_* files are
%% id1,study1 - brain surface 1
%% id2,study2 - brain surface 2
%% useright - load the right (1) or left (0) hemisphere (default: 0)
%%
%% corr - correspondence map ( corr(i) gives the corresponding point index
%% on mesh 2 of the ith point on mesh 1 )
%%

function corr = loadfreesurfermatching(pathname,id1,study1,id2,study2,useright)

if nargin<6; useright=0; end;

if useright
    filename = [pathname '/MT_' num2str(id1,'%03d') '/' num2str(study1,'%02d') '_recon/rh.correspondence.MT_' num2str(id2,'%03d') '.label'];
else
    filename = [pathname '/MT_' num2str(id1,'%03d') '/' num2str(study1,'%02d') '_recon/lh.correspondence.MT_' num2str(id2,'%03d') '.label'];
end

[idx corr] = textread(filename,  '%f %f', 'commentstyle','shell','headerlines',2);
    
corr = sortrows([idx corr]+1,1); clear idx; % sort first colum, add 1 since text files start indexing at 0
corr = corr(:,2);                           % get second column

end
