%% loadmesh.m
%% Herve Lombaert, April 28th, 2010
%%
%% Load a Brain Mesh
%%
%% filename - either
%%            3) .wrl file, containing a structure with
%%               vertices, and faces
%%            2).mat file, containing a structure with
%%               lh_white.vertices, lh_white.faces
%%               rh_white.vertices, rh_white.faces
%% useright - if loading a .mat file, load the right hemisphere (1) or the
%%            left hemisphere (0: default)
%%
%% X - Nx3 matrix containing the 3D coordinates of the mesh vertices
%% F - Mx3 matrix containing triangle definition (one row is [indexA,indexB,indexC])
%%

function [X,F] = loadmesh(filename, useright)

extension = filename((end-3):end);

if strcmp(extension,'.mat')
    
    %% Loading MAT file
    if nargin<2; useright = 0; end;
    
    if useright
        mesh = load(filename, 'rh_white');
        X    = mesh.rh_white.vertices;
        F    = mesh.rh_white.faces;
    else
        mesh = load(filename, 'lh_white');
        X    = mesh.lh_white.vertices;
        F    = mesh.lh_white.faces;
    end    

else if strcmp(extension,'.wrl')
        
    %% Loading WRL file
    [X,F] = read_wrl(filename); X=X'; F=F';
    
end; end
end
