%% loadparcellations.m
%% Herve Lombaert, April 28th, 2010
%%
%% Load all parcellation definition for one dataset
%%
%% pathname - where all the destrieux2005_parcellation_lh-XXX.label files are
%% nbpoints - number of mesh nodes
%% parcels  - what parcels to load (tells what files to read)
%% useright - load right parcels (left by default)
%%
%% P - Npoints x Nparcels, P(i,j) tells if the ith mesh node belong to parcel j
%%

function P = loadparcels(pathname, nbpoints, parcels, useright)

if nargin<4; useright=0; end;

P = zeros(nbpoints,size(parcels,2));

for i=1:size(parcels,2)

    if useright; leftright = 'rh'; else leftright = 'lh'; end;
    filename = [pathname '/destrieux2005_parcellation_' leftright '-' num2str(parcels(i),'%03d') '.label'];
    if ~exist(filename); disp([filename ' does not exist']); continue; end
    
    idx = textread(filename, '%f %*f %*f %*f %*f', 'commentstyle','shell','headerlines',2);
    idx = idx + 1;  % index begins with 0 in files
    P(idx,i) = 1; % mark parcel vertices
    
end

end
