% THRESHOLDCURVATURE Threshold curvature
%
%    K=THRESHOLDCURVATURE(K) threshold curvatures so the histogram of the
%    output is nicely spread out

% author:  Herve Lombaert
% contact: herve.lombaert@gmail.com
% date:    April 23rd, 2011

function K = thresholdcurvature(K)

% Find threshold
s  = max(K);     % initial standard deviation
sp = 2*s;        % initial previous stddev
max_iter = 20;   % max iteration
iter     = 0;
while sp/s>1.01 && iter<max_iter % wait convergence
    sp = s; iter = iter+1;
    [n,x]=hist(K(abs(K)<3*s),100);                     % get histogram
    m = n*x' / sum(n);                                 % mean
    s = sqrt(bsxfun(@minus,x,m).^2 * n' / (sum(n)-1)); % stddev
end

% Threshold
K(K> 3*s) =  3*s;
K(K<-3*s) = -3*s;
