%% runme.m
%% Herve Lombaert, April 28th, 2010
%%
%% Main file for brain matching using spectral correspondence
%%
%% Loads two hemispheres, matches them, and display them with an
%% interactive correspondence marker (clicking on top hemisphere will
%% display the corresponding point on the bottom hemisphere)
%%

clear all;
clf;
    
addpath('smooth');
addpath('matching');
addpath('misc');
addpath('loading');

addpath('cpd');
addpath('cpd/FGT');
addpath('cpd/mex');
addpath('cpd/Nonrigid');
addpath('cpd/Rigid');
addpath('cpd/utils');

addpath('misc/curvature');

%% Parameters
pathname     = './data'; % where the data is
mesh1        = [pathname '/horse-gallop-01.obj'];
mesh2        = [pathname '/horse-gallop-03.obj'];
usecurvature      = 0;  % use curvature as extra coordinate
node_usecurvature = 0;  % use curvature as node weight
edge_usecurvature = 0;  % use curvature as edge weight
do_print          = 0;  % output in image
do_avi            = 0;  % create rotating video

ne = 7; % number of eigenvector to use (not counting the first null eigenvector)

% Load mesh 1
[X1,F1] = readobj(mesh1);
X1 = [X1(:,1),-X1(:,3),X1(:,2)]; % rotate mesh
if usecurvature||node_usecurvature||edge_usecurvature; data.Curvature = curvature(X1,F1); end; % Recompute curvature
if usecurvature;      C1 = data.Curvature;        else C1 = []; end;
if node_usecurvature; node_C1 = data.Curvature;   else node_C1 = []; end;
if edge_usecurvature; edge_C1 = data.Curvature;   else edge_C1 = []; end;

% Load mesh 2
[X2,F2] = readobj(mesh2);
X2 = [X2(:,1),-X2(:,3),X2(:,2)]; % rotate mesh
if usecurvature||node_usecurvature||edge_usecurvature; data.Curvature = curvature(X2,F2); end; % Recompute curvature
if usecurvature;      C2 = data.Curvature;        else C2 = []; end;
if node_usecurvature; node_C2 = data.Curvature;   else node_C2 = []; end;
if edge_usecurvature; edge_C2 = data.Curvature;   else edge_C2 = []; end;

%% Match brain surfaces
disp(['Matching ' mesh1 ' with ' mesh2]);

% Main call for the matching method
% just provide X1,F1 and X2,F2, two meshes, S1,S2,H1,H2,C1,C2 are optional (sulcal depth, cortical thickness, and curvature)
option.ne          = ne;          % number of eigenvectors
option.beta        = 1.2;         % EX factor
option.gamma       = 0.2;         % EW factor
option.alpha       = 1.0;         % NW factor
option.EX1         = [C1];  % Extra features on mesh 1
option.EX2         = [C2];  % Extra features on mesh 2
option.EW1         = [C1];
option.EW2         = [C2];
option.NW1         = exp([C1]); % must be positive
option.NW2         = exp([C2]); % must be positive
[corr_SC,~,E1,E2]  = match(X1,X2,F1,F2,option);

%% Display Correspondences
displaycorrespondence(X1,X2,F1,F2,corr_SC,E1,E2);
disp('*** Click on the top mesh of the figure to find its match below ***');

%% Rotate cortices
set(gcf,'Color','white');

if do_avi
    aviobj = avifile('correspondence.avi','compression','None', 'fps',10);
    fig = figure(1);

    for i=0:.05:2
        repositioncam(X1,i-0.5);
        zoom(2); drawnow;
        pause(0.5);
        aviobj = addframe(aviobj,getframe(fig));
    end
    
    aviobj = close(aviobj);
    
    %system('/opt/local/bin/mencoder correspondence.avi -ovc lavc -lavcopts vcodec=wmv1 -nosound -o test.avi && mv test.avi correspondence.avi');
end


